/* 
 * event.h -- header file for event handling routines
 *
 * rlc -- Reliable Multicast data Distribution Protocol
 * 
 * (C) 1996, 1997, 1998 Luigi Rizzo and Lorenzo Vicisano
 *     (luigi@iet.unipi.it, vicisano@cs.ucl.ac.uk)
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Luigi Rizzo,
 *      Lorenzo Vicisano and other contributors.
 * 4. Neither the name of the Authors nor the names of other contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
#ifndef RLC__EVENT_H
#define RLC__EVENT_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <assert.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include "rlcif.h"
#include "compat.h"

#define MAX_CHAN 32

typedef struct _timit_t {
    struct _timit_t *next;
    int id;				/* event id. */
    int flags;				/* flags */
    int (*t_f)(void *);			/* function to call */
    void *t_p;				/* function parameter list */
    ui32 delta;				/* delta t (usec) */
} timit_t;

typedef struct _evlist_t {
    int io_chan[MAX_CHAN];		/* fd, -1 if empty */
    int io_id[MAX_CHAN];		/* channel id. */
    ui16 io_ch_num;			/* number of active channels */
    int (*io_f[MAX_CHAN])(void *);	/* function to call */
    void *io_p[MAX_CHAN];		/* function parameter list */
    int (*def_f)(void *);		/* default function */
    void *def_p;			/* default parameters */
    fd_set in_s;
    fd_set out_s;
    fd_set err_s;
    timit_t *timevent;			/* timevent list */
    struct timeval lasttime;
    struct timeval default_timeout;
    ui32 late;				/* we are scheduling the current
					   event late usec late! */
    int selected;			/* the last selected fd */
} evlist_t;

extern void *rlc_events;

void *rlc_createevlist_();
int rlc_getevent_(void *e, int blocking);
int rlc_insertchan_(void *e, int chan, int id, CALLBACK f, void *p);
void rlc_deletechan_(void *e, int chan);
int rlc_settimer_(void *e, int id, unsigned long delta, CALLBACK f,
	void *p, int real);
void rlc_set_default_timeout_(void *e, unsigned long delta);
int rlc_showselected_ (void *e);
int rlc_deletetimer_(void *e, int id);
int rlc_poolchan_(void *vp, int *rid, void **rp);

/**
 ** event tags and some function prototype are defined in rlcif.h
 **/

#endif  /* RLC__EVENT_H */
