/* 
 * layers.h -- headers for layers routines.
 *
 * rlc -- Reliable Multicast data Distribution Protocol
 * 
 * (C) 1996, 1997, 1998 Luigi Rizzo and Lorenzo Vicisano
 *     (luigi@iet.unipi.it, vicisano@cs.ucl.ac.uk)
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Luigi Rizzo,
 *      Lorenzo Vicisano and other contributors.
 * 4. Neither the name of the Authors nor the names of other contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
#ifndef RLC__LAYERS_H
#define RLC__LAYERS_H

#include <sys/types.h>
#include <netinet/in.h>
#include "rlc.h"
#include "rlcif.h"

#define TX_INPROBE	'\1'		/* this relates to application API */

#define IS_OVER 0xffffffff

typedef struct _lay_d {
    ui16 pn;
    int nc;			/* number of layers */
#ifdef ONE_SEND_SOCK
    int chan;
    struct sockaddr_in saddrs[MAX_LAYERS];
#else
    int chs[MAX_LAYERS];	/* sockets */
#endif
    ui32 pu;			/* period unit: 1/f_max, in usec */
    ui16 idxs[MAX_LAYERS];	/* next index in this layer */
    ui32 rates[MAX_LAYERS];	/* tx rate, actually period in pu_s */
    ui32 left[MAX_LAYERS];	/* time left to next send */
    ui32 gap[MAX_LAYERS];	/* gap period after a burst */
    int  cyc_l[MAX_LAYERS];	/* cycle length in pkts */
    int  bur_l[MAX_LAYERS];	/* burst length in pkts */
    int W, Y, P;
    int inc_todo;
    ui32 now;			/* debugging only */
} lay_d;

ui32 l_change_rate(ui32 rate, lay_d *l);
void setup_layers(ui32 rate, lay_d *l, int W, int Y, int P);
int setup_l_sock(lay_d *l, addr_list_t *addr, n16 port, int ttl);
ui32 send_layers();
#if 1
ui32    global_index (ui16 lsn, ui16 pn, int lay, int curr_lay, int inbu);
void    from_global_index (ui32 gidx, ui16 *lsn, ui16 *pn, int *lay,
        int curr_lay, int inbu);
int is_first_burst (int lay, int curr_lay, char f);
int is_first_non_burst (int lay, int curr_lay, char f);
int is_burst_ending (char f);
int is_in_burst (char f);
int up_synch_burst (int lay, int curr_lay, char f);

#else
int is_first_burst (ui16 lsn, ui16 pn, int lay, int curr_lay, int W, int Y);
int is_first_non_burst (ui16 lsn, ui16 pn, int lay, int curr_lay, int W, int Y);
int is_burst_ending (ui16 lsn, ui16 pn, int lay, int curr_lay, int W, int Y);
int is_in_burst (ui16 pn, int W, int Y);
ui16 howmany_burst (ui16 pn, int W);
ui32 global_index (ui16 lsn, ui16 pn, int lay, int curr_lay, int W, int Y);
void from_global_index (ui32 gidx, ui16 *lsn, ui16 *pn, int *lay,
        int curr_lay, int W, int Y);
#endif

#endif  /* RLC__LAYERS_H */
