/*
 * se.c -- dummy packet multicast sender.
 *
 * This file is part of
 * 
 * rlc -- Reliable Multicast data Distribution Protocol
 * 
 * (C) 1996, 1997, 1998 Luigi Rizzo and Lorenzo Vicisano
 *     (luigi@iet.unipi.it, vicisano@cs.ucl.ac.uk)
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Luigi Rizzo,
 *      Lorenzo Vicisano and other contributors.
 * 4. Neither the name of the Authors nor the names of other contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#include <math.h>
#include "rlc.h"

int data_pkt_len = 1024;
void *events;	/* callback attach. and timers */
char data[MAX_PAYLOAD_SIZE];
int datas;
ui32 period = 1;


int
se_send(int *datas)
{
#if 0
  int i, j;
  double a;

    printf("%u\n", timestamp_u());
    fflush(stdout);
    
    j = drand48() * 10000;
    for (i=0; i<j; i++)
	a = pow (3., 56.);
#endif
    if (send(*datas, data, data_pkt_len, 0) == -1) {
	 perror("send");
    }

    rlc_settimer(events, 13, period, (CALLBACK)se_send, datas, 0);
    DEB(fprintf(stderr, "--- sending \n"))
    return 0;
}

void
badcmdline(char *arg)
{
    fprintf(stderr,
	"Bad argument %s\nCommand line:\n\t rlcd [options]\n"
	"options:\n"
	"\t-r <data rate (pkts/s)>\n"
	"\t-d <M-cast data address/port>\n"
	"\t-l <packet size (bytes)>\n"
	"\t-t <sender ttl>\n", arg);
    exit(1);
}

int
main(int argc, char *argv[])
{
  int i;

  char dataaddr[ADDRLEN];
  n16 dport = 0;
  ui8 ttl=TTL;
  ui32 daddr;

    extern char *optarg;
    extern int optind;

    /* parse command line... */
    while ((i = getopt(argc,argv,"t:d:l:")) != EOF) {
	switch(i) {
	case 'd': /* data addr/port */
	    if(set_link_addr(optarg, dataaddr, ADDRLEN, &dport, "data")<0)
		badcmdline(optarg);
	    break;
	case 't': /* TTL */
	    ttl = atoi(optarg);
	    if (ttl <= 0 || ttl > 255) badcmdline(optarg);
	    break;
	case 'l': /* data packet size */
	    data_pkt_len = atoi(optarg);
	    if (data_pkt_len < 16 /* XXX */ ||
		data_pkt_len > MAX_PAYLOAD_SIZE) badcmdline(optarg);
	case 'r': /* data rate pkts/s */
	    period = atoi(optarg);
	    break;
	}
    }
    if (dport == 0)
	set_link_addr(DEFAULT_DPORT, dataaddr, ADDRLEN, &dport, "data");
    fprintf(stderr, "using ttl %d \n", ttl);
    if (period > 0) {
	fprintf(stderr, "rate %d pkts/s (%.3f kbit/s)\n", period,
	    period*data_pkt_len*8/1024.);
	period = 1e6/period;
    } else {
	fprintf(stderr, "Wrong data rate\n");
	exit (1);
    }
    argc -= optind;
    argv += optind;

    events = rlc_createevlist();

    daddr = inet_addr(dataaddr) ;

    datas = openssock(daddr, dport, ttl /* TTL */, 0);

    rlc_settimer(events, 13, period, (CALLBACK)se_send, &datas, 1);

    while (rlc_getevent(events, 1)!=EV_NO_PENDING);

    exit(0);
}
